#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 Connect to and Export from DB

 @author East Japan Accounting Center Co., Ltd

 (c)2019 The University of Aizu
 This software is released under the New BSD License.
"""
from logging import getLogger

import pymongo
from pymongo.errors import ConnectionFailure, InvalidName, InvalidOperation

logger = getLogger(__name__)

# Index
INDEX_DATETIME = "datetime.datetime_1"


class MongoDB:
    """
    MongoDB operation(Connection, File export, ...)
    """

    def __init__(self, host, port, db_name, collection_name=""):
        self._client = None
        self._db = None
        self._host = host
        self._port = port
        self._db_name = db_name
        self._collection_name = collection_name

    def _connection(self, host="localhost", port=27017):
        """
        Connect to specified DB

        Parameters
        ----------
        host: string(Optional, Default: localhost)
            Host name on which to connect
        port: int(Optional, Default: 27017)
            Port number on which to connect

        Returns
        -------
        DB instance
        """

        try:
            self._client = pymongo.MongoClient(host=host, port=port)
        except ConnectionFailure:
            logger.error("Unable to connect DB. Please confirm the HOST and PORT.")

        return self._client

    def _export(self, db_name, collection_name, exp_path, query):
        """
        Data export

        Parameters
        ----------
        db_name: string
            DB name
        collection_name: string
            Collection name
        ext_path: string
            CSV export file path
        query: dictionary
            Query
        """

        df = self._get_document(db_name, collection_name, query)
        if df is not None:
            self._exp_csv(exp_path, df)

    def _get_count_document(self, db_name, collection_name, query):
        """
        Data corresponding to the query

        Parameters
        ----------
        db_name: string
            DB name
        collection_name: string
            Collection name
        query: dictionary
            Query

        Returns
        -------
        doc_counts: int
            Document count(s)
        """

        self._db_name = db_name
        self._collection_name = collection_name
        doc_counts = 0

        # Do NOT export if the DB, collection are not specified
        if len(db_name) == 0 and len(collection_name) == 0:
            return doc_counts

        # Querying for more than one document
        try:
            self._db = self._client[db_name]
            co = self._db[collection_name]
            doc_counts = co.count_documents(filter=query)
        except Exception:
            logger.error("Unable to get the document.")

        return doc_counts

    def _get_document(self, db_name, collection_name, query):
        """
        Data corresponding to the query

        Parameters
        ----------
        db_name: string
            DB name
        collection_name: string
            Collection name
        query: dictionary
            Query

        Returns
        -------
        df: pandas.core.frame.DataFrame
            Dataframe
        """

        self._db_name = db_name
        self._collection_name = collection_name
        doc = None

        # Do NOT export if the DB, collection are not specified
        if len(db_name) == 0 and \
           len(collection_name) == 0:
            return doc

        # Querying for more than one document
        try:
            self._db = self._client[db_name]
            co = self._db[collection_name]
            doc = co.find(filter=query)
        except InvalidName:
            logger.error("Invalid DB or collection. Please confirm.")
        except Exception:
            logger.error("Unable to get the document.")

        return doc

    def _insert_document(self, db_name, collection_name, doc):
        """
        Insert the one document in the specified DB

        Parameters
        ----------
        db_name: string
            DB name
        collection_name: string
            Collection name
        doc: dictionary
            Document

        Returns
        -------
        True: Success False: Failure
        """

        self._db_name = db_name
        self._collection_name = collection_name
        rtn = False

        # Do NOT export if the DB, collection are not specified
        if len(db_name) == 0 or len(collection_name) == 0:
            logger.error("DB param is empty.")
            return rtn

        # Insert the one document in the specified DB
        try:
            self._db = self._client[db_name]
            co = self._db[collection_name]
            ins_one_result = co.insert_one(doc)
            if ins_one_result.acknowledged:
                rtn = True
        except InvalidOperation:
            logger.error("Invalid DB operation. Please confirm.")
        except Exception as e:
            logger.error(e)
            logger.error("Unable to insert the document.")

        return rtn

    def _exp_csv(self, exp_path, df):
        """
        Export as a CSV file

        Parameters
        ----------
        ext_path: string
            CSV export file path
        df: pandas.core.frame.DataFrame
            Dataframe

        Returns
        -------
        rtn: boolean
            Export result
            True: Success
            False: Failure
        """

        rtn = False

        if len(exp_path) == 0:
            logger.error("Export filepath is empty.")
            return rtn

        try:
            df.to_csv(exp_path, index=False)
            rtn = True
        except Exception:
            logger.error("Export file create error")

        return rtn

    def _close(self):
        """
        MongoDB client close
        """

        if self._client:
            self._client.close()
