#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 rdr_sub_main.py

 @author East Japan Accounting Center Co., Ltd

 (c)2021 The University of Aizu
 This software is released under the New BSD License.
"""

import datetime
import json
import logging
import os
import queue
import time
import threading

import mongodb
import postgres
from rdr_pg_insert_for_robot import PointCloud2Message

JST = datetime.timezone(datetime.timedelta(hours=+9), 'JST')

APP_LOG_ROOT_DIR = "log/"

curr_datetime = datetime.datetime.now()
curr_date = "{0:04d}{1:02d}{2:02d}".format(curr_datetime.year, curr_datetime.month, curr_datetime.day)
curr_time = "{0:02d}{1:02d}{2:02d}".format(curr_datetime.hour, curr_datetime.minute, curr_datetime.second)
log_root_dir = (APP_LOG_ROOT_DIR + curr_date + "/")
if os.path.isdir(log_root_dir) is False:
    os.makedirs(log_root_dir)
log_filename = "{0}{1}_{2}_{3}{4}".format(log_root_dir, os.path.splitext(os.path.basename(__file__))[0], curr_date, curr_time, ".log")

logger = logging.getLogger(__name__)

st_log_fmt = "%(asctime)s - %(levelname)s - %(message)s"
stream_handler = logging.StreamHandler()
stream_handler.setLevel(logging.INFO)
stream_handler.setFormatter(logging.Formatter(st_log_fmt))

fl_log_fmt = "%(asctime)s %(name)s - %(levelname)s - [%(filename)s:%(lineno)d] - %(message)s"
file_handler = logging.FileHandler(filename=log_filename, encoding="utf-8")
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(logging.Formatter(fl_log_fmt))

logging.basicConfig(level=logging.NOTSET, handlers=[stream_handler, file_handler])


lock = threading.Lock()


class LifeTime:
    thread_life_time = False

    @classmethod
    def is_life_time(cls):
        return cls.thread_life_time

    @classmethod
    def set_life_time_activation(cls):
        cls.thread_life_time = True

    @classmethod
    def set_life_time_deactivation(cls):
        cls.thread_life_time = False


def pcd_thread(pcd_queue: queue.Queue, pg, cur, error_queue: queue.Queue):
    """
    Create PCD file.

    Parameters
    ----------
    pcd_queue
        PCD Queue
    pg
        PostgreSQL instance
    cur
        PostgreSQL cursor
    error_queue
        Error Queue

    Returns
    -------
    """

    try:
        while LifeTime.is_life_time():
            if not pcd_queue.empty():
                with lock:
                    doc = pcd_queue.get(block=True, timeout=None)
                PointCloud2Message.point_cloud_insert(doc, pg, cur)
                pcd_queue.task_done()
                if pcd_queue.empty():
                    LifeTime.set_life_time_deactivation()
                else:
                    time.sleep(1)
    except Exception as e:
        error_queue.put(True)
        logger.error(e)


def _is_json_file_parse(path):
    """
    Load the json file.

    Parameters
    ----------
    path
        json file path

    Returns
    -------
    Parse result
        True: Success False: Failure
    """

    result = False
    json_dict = {}

    try:
        with open(path) as f:
            json_dict = json.load(f)
            if "" not in json_dict.values():
                result = True
            else:
                logger.error("Param is empty.Please confirm.")
    except IOError:
        logger.error("Unable to read setting file.")

    return result, json_dict


def main():
    pcd_queue = queue.Queue()
    error_queue = queue.Queue()

    result, db_param = _is_json_file_parse("./db_settings.json")
    if result is False:
        return

    mongo = mongodb.MongoDB(db_param['host'], db_param['port'], db_param['db_name'], db_param['db_tbl'])
    pg = postgres.PostgreSQL(db_param['host_2nd'], db_param['port_2nd'], db_param['db_name_2nd'], db_param['db_tbl_2nd'], db_param['db_user_2nd'], db_param['db_pass_2nd'])
    try:
        with mongo._connection() as mongo_client, pg.get_connection() as pg_client:
            if mongo_client and pg_client:
                with pg_client.cursor() as cur:
                    today = datetime.date.today()
                    query = {"time": {"$gte": str(today - datetime.timedelta(days=1)) + "T00:00:00.000", "$lt": str(today) + "T00:00:00.000"}}
                    doc_cnt = mongo._get_count_document(db_param["db_name"], db_param["db_tbl"], query)
                    if doc_cnt > 0:
                        logger.info("PCD documents: %d", doc_cnt)
                        docs = mongo._get_document(db_param["db_name"], db_param["db_tbl"], query)
                        pcd_thread_ins = threading.Thread(target=pcd_thread, args=(pcd_queue, pg, cur, error_queue), daemon=True)
                        LifeTime.set_life_time_activation()
                        pcd_thread_ins.start()

                        for doc in docs:
                            pcd_queue.put(doc)

                        pcd_thread_ins.join()
                        logger.info("PCD file create complete")
                    else:
                        logger.info("PCD documents are not found.")
    except KeyboardInterrupt:
        LifeTime.set_life_time_deactivation()


if __name__ == "__main__":
    main()
